@@ -199,6 +199,35 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 		return( 0.0 );
 	}
 
+	lgth1 = strlen( seq1[0] );
+	lgth2 = strlen( seq2[0] );
+
+	if( lgth1 <= 0 || lgth2 <= 0 )
+	{
+		fprintf( stderr, "WARNING (g11): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
+	}
+
+#if 1
+	if( lgth1 == 0 && lgth2 == 0 )
+		return( 0.0 );
+
+	if( lgth1 == 0 )
+	{
+		seq1[0][lgth2] = 0;
+		while( lgth2 ) seq1[0][--lgth2] = '-';
+//		fprintf( stderr, "seq1[0] = %s\n", seq1[0] );
+		return( 0.0 );
+	}
+
+	if( lgth2 == 0 )
+	{
+		seq2[0][lgth1] = 0;
+		while( lgth1 ) seq2[0][--lgth1] = '-';
+//		fprintf( stderr, "seq2[0] = %s\n", seq2[0] );
+		return( 0.0 );
+	}
+#endif
+
 
 	wm = 0.0;
 
@@ -209,15 +238,6 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	}
 
 
-	lgth1 = strlen( seq1[0] );
-	lgth2 = strlen( seq2[0] );
-
-
-
-	if( lgth1 <= 0 || lgth2 <= 0 )
-	{
-		fprintf( stderr, "WARNING (g11): lgth1=%d, lgth2=%d\n", lgth1, lgth2 );
-	}
 
 	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
 	{
