@@ -210,6 +210,28 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 	float fpenalty = (float)penalty;
 	float fpenalty_ex = (float)penalty_ex;
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 > 0 && orlgth2 > 0 )
+		{
+			orlgth1 = 0;
+			orlgth2 = 0;
+			free( mseq1 );
+			free( mseq2 );
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+
+			FreeFloatVec( m );
+			FreeIntVec( mp );
+
+			FreeCharMtx( mseq );
+
+		}
+		return( 0.0 );
+	}
 
 
 	if( orlgth1 == 0 )
