@@ -5,7 +5,7 @@
 #define OUTGAP0TRY 0
 #define DEBUG 0
 #define XXXXXXX    0
-#define USE_PENALTY_EX  0
+#define USE_PENALTY_EX  1
 #define FASTMATCHCALC 1
 
 
@@ -924,6 +924,7 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float ogcp1va;
 
 
+
 	if( seq1 == NULL )
 	{
 		if( orlgth1 )
@@ -1024,6 +1025,7 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	{
 		int ll1, ll2;
 
+
 		if( orlgth1 > 0 && orlgth2 > 0 )
 		{
 			FreeFloatVec( w1 );
@@ -1205,12 +1207,24 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 #if OUTGAP0TRY
 	else
 	{
+		fprintf( stderr, "offset = %d\n", offset );
 		for( j=1; j<lgth2+1; j++ )
 			currentw[j] -= offset * j / 2.0;
 		for( i=1; i<lgth1+1; i++ )
 			initverticalw[i] -= offset * i / 2.0;
 	}
 #endif
+#if 0
+	fprintf( stderr, "\n   " );
+	for( j=0; j<lgth2+1; j++ ) fprintf( stderr, "    %c ", seq2[0][j] );
+	fprintf( stderr, "\n%c ", seq1[0][0]  );
+	for( j=0; j<lgth2+1; j++ )
+	{
+		fprintf( stderr, "%5.0f ", currentw[j] );
+	}
+	fprintf( stderr, "\n"  );
+#endif
+
 
 	for( j=1; j<lgth2+1; ++j ) 
 	{
@@ -1293,7 +1307,15 @@ fprintf( stderr, "\n" );
 #endif
 		currentw[0] = initverticalw[i];
 
-
+#if 0
+		fprintf( stderr, "%c ", seq1[0][i] );
+		for( j=0; j<lgth2+1; j++ )
+		{
+			fprintf( stderr, "%5.0f ", currentw[j] );
+		}
+		fprintf( stderr, "\n"  );
+#endif
+	
 		mi = previousw[0] + ogcp2[1]; mpi = 0;
 		ijppt = ijp[i] + 1;
 		mjpt = m + 1;
