@@ -1,8 +1,9 @@
 #include "mltaln.h"
 
+
 void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **aln2, int alloclen, char alg ) // n1 ha allgap
 {
-	int i, newlen;
+	int i, j, newlen;
 	double *effarr0, *effarr2;
 	float dumfl;
 	double eff;
@@ -12,10 +13,10 @@ void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **
 	commongappick( n0, aln0 );
 	commongappick( n2, aln2 );
 
-
 	eff = 1.0 / (double)n0; for( i=0; i<n0; i++ ) effarr0[i] = eff;
 	eff = 1.0 / (double)n2; for( i=0; i<n2; i++ ) effarr2[i] = eff;
 
+	newgapstr = "-";
 	if( alg == 'M' )
 		MSalignmm( aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 ); //outgap=1??
 	else
@@ -27,6 +28,33 @@ void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **
 	aln1[0][i] = 0;
 	for( i=1; i<n1; i++ ) strcpy( aln1[i], aln1[0] );
 
+	for( j=0; j<newlen; j++ )
+	{
+//		fprintf( stderr, "j=%d\n", j );
+		for( i=0; i<n0; i++ )
+		{
+			if( aln0[i][j] != '-' ) break;
+		}
+		if( i == n0 ) 
+		{
+			for( i=0; i<n1; i++ ) 
+			{
+				if( aln1[i][j] != '-' ) break;
+			}
+		}
+		else i = -1;
+
+		if( i == n1 ) 
+		{
+			for( i=0; i<n1; i++ ) aln1[i][j] = '=';
+		}
+	}
+#if 0
+	fprintf( stderr, "in profilealignment,\n" );
+	for( i=0; i<n0; i++ ) fprintf( stderr, "\n>aln0[%d] = \n%s\n", i, aln0[i] );
+	for( i=0; i<n1; i++ ) fprintf( stderr, "\n>aln1[%d] = \n%s\n", i, aln1[i] );
+	for( i=0; i<n2; i++ ) fprintf( stderr, "\n>aln2[%d] = \n%s\n", i, aln2[i] );
+#endif
 
 	free( effarr0 );
 	free( effarr2 );
@@ -41,18 +69,17 @@ void eq2dash( char *s )
 	}
 }
 
-void findnewgaps( int n, char **seq, int *gaplen )
+void findnewgaps( int n, int rep, char **seq, int *gaplen )
 {
 	int i, pos, len;
-	len = strlen( seq[0] );	
 
-//	fprintf( stderr, "seq[0] = %s\n", seq[0] );
-	for( i=0; i<len; i++ ) gaplen[i] = 0;
+	len = strlen( seq[0] );	
+	for( i=0; i<len; i++ ) gaplen[i] = 0; // calloc de shokika sareteirukara hontou ha iranai
 	
 	pos = 0;
 	for( i=0; i<len; i++ )
 	{
-		if( seq[0][i] == '=' ) 
+		if( seq[rep][i] == '=' ) 
 		{
 //			fprintf( stderr, "Newgap! pos = %d\n", pos );
 			gaplen[pos]++;
@@ -96,6 +123,7 @@ void adjustgapmap( int newlen, int *gapmap, char *seq )
 	int newlen1 = newlen+1;
 	int *tmpmap;
 
+
 	tmpmap = AllocateIntVec( newlen+2 );
 	j = 0;
 	pos = 0;
@@ -117,17 +145,36 @@ void adjustgapmap( int newlen, int *gapmap, char *seq )
 	free( tmpmap );
 }
 
-void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int *gapmap, int alloclen, char alg )
+static void strncpy0( char *s1, char *s2, int n )
+{
+	while( n-- ) *s1++ = *s2++;
+	*s1 = 0;
+}
+
+static int countnogaplen( int *gaplen, int *term )
+{
+	int v = 0;
+	while( gaplen < term )
+	{
+		if( *gaplen++ == 0 ) v++;
+		else break;
+	}
+	return( v );
+}
+
+void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int *gapmap, int alloclen, char alg, char gapchar )
 {
 	int *mar;
 	char *gaps;
 	char *cptr;
-	int i, j, k, len, rep, len0;
+	int i, j, k, len, rep, len0, lp, blocklen;
 	char **mseq2, **mseq0, **mseq1;
-	char **aseq;
+	char **aseq, *newchar;
 	int ngroup2, ngroup0, ngroup1;
 	int *list0, *list1, *list2;
 	int posin12, gapshift, newpos;
+	int mlen1, mlen0, mlen2;
+	int tmptmptmpmark = 0;
 
 	mar = calloc( njob, sizeof( int ) );
 	list0 = calloc( njob, sizeof( int ) );
@@ -166,13 +213,14 @@ void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex
 		if( mar[i] == 0 ) 
 		{
 			list0[ngroup0] = i;
+//			fprintf( stderr, "inserting new gaps to %d\n", i );
 			ngroup0++;
 		}
 	}
 	list0[ngroup0] = list1[ngroup1] = list2[ngroup2] = -1;
 	if( ngroup0 == 0 )
 	{
-		fprintf( stderr, "Nothing to do\n" );
+//		fprintf( stderr, "Nothing to do\n" );
 		free( mar );
 		free( list0 );
 		free( list1 );
@@ -199,77 +247,149 @@ void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex
 	aseq = AllocateCharMtx( njob, alloclen );
 	gaps = calloc( alloclen, sizeof( char ) );
 
+
 	for( i=0; i<njob; i++ ) aseq[i][0] = 0;
+	newpos = 0;
 	posin12 = 0;
+//	fprintf( stderr, "\ngaplen[] = \n" );
+//	for(i=0; i<len0; i++ ) fprintf( stderr, "%d ", gaplen[i] );
+//	fprintf( stderr, "\n" );
+
 	for( j=0; j<len0; j++ )
 	{
+//		fprintf( stderr, "\nj=%d, gaplen[%d]=%d\n", j, j, gaplen[j] );
 		if( gaplen[j] )
 		{
+//			fprintf( stderr, "j=%d GAP!\n", j );
+			tmptmptmpmark = 1;
 			for( i=0; i<ngroup0; i++ ) mseq0[i][0] = 0;
 			for( i=0; i<ngroup1; i++ ) mseq1[i][0] = 0;
 			for( i=0; i<ngroup2; i++ ) mseq2[i][0] = 0;
+			mlen0 = mlen1 = mlen2 = 0;
 
 			gapshift = gaplen[j];
 			cptr = gaps;
-			while( gapshift-- ) *cptr++ = '-';
+			while( gapshift-- ) *cptr++ = gapchar;
 			*cptr = 0;
 			gapshift = gaplen[j];
 
-			for( i=0; i<ngroup0; i++ ) strncat( mseq0[i], gaps, gapshift );
-			for( i=0; i<ngroup1; i++ ) strncat( mseq1[i], seq[list1[i]]+posin12, gapshift );
-			for( i=0; i<ngroup2; i++ ) strncat( mseq2[i], seq[list2[i]]+posin12, gapshift );
+			for( i=0; i<ngroup0; i++ ) strncpy0( mseq0[i]+mlen0, gaps, gapshift );
+			for( i=0; i<ngroup1; i++ ) strncpy0( mseq1[i]+mlen1, seq[list1[i]]+posin12, gapshift );
+			for( i=0; i<ngroup2; i++ ) strncpy0( mseq2[i]+mlen2, seq[list2[i]]+posin12, gapshift );
 			posin12 += gapshift;
+			mlen0 += gapshift;
+			mlen1 += gapshift;
+			mlen2 += gapshift;
 
 			gapshift = gapmap[posin12];
 //			fprintf( stderr, "gapmap[%d] kouho = %d\n", posin12, gapmap[posin12] );
 
 
-			for( i=0; i<ngroup0; i++ ) strncat( mseq0[i], seq[list0[i]]+j, gapshift );
-			for( i=0; i<ngroup1; i++ ) strncat( mseq1[i], seq[list1[i]]+posin12, gapshift );
-			for( i=0; i<ngroup2; i++ ) strncat( mseq2[i], seq[list2[i]]+posin12, gapshift );
+			for( i=0; i<ngroup0; i++ ) strncpy0( mseq0[i]+mlen0, seq[list0[i]]+j, gapshift );
+			for( i=0; i<ngroup1; i++ ) strncpy0( mseq1[i]+mlen1, seq[list1[i]]+posin12, gapshift );
+			for( i=0; i<ngroup2; i++ ) strncpy0( mseq2[i]+mlen2, seq[list2[i]]+posin12, gapshift );
+			mlen0 += gapshift;
+			mlen1 += gapshift;
+			mlen2 += gapshift;
 #if 0
 			for( i=0; i<ngroup0; i++ ) fprintf( stderr, "### mseq0[%d] = %s\n", i, mseq0[i] );
 			for( i=0; i<ngroup1; i++ ) fprintf( stderr, "### mseq1[%d] = %s\n", i, mseq1[i] );
 			for( i=0; i<ngroup2; i++ ) fprintf( stderr, "### mseq2[%d] = %s\n", i, mseq2[i] );
 #endif
 
-			if( gapshift ) profilealignment( ngroup0, ngroup1, ngroup2, mseq0, mseq1, mseq2, alloclen, alg );
+			if( gapshift ) 
+				profilealignment( ngroup0, ngroup1, ngroup2, mseq0, mseq1, mseq2, alloclen, alg );
 
 			j += gapshift;
 			posin12 += gapshift;
 
-			for( i=0; i<ngroup0; i++ ) strcat( aseq[list0[i]], mseq0[i] );
-			for( i=0; i<ngroup1; i++ ) strcat( aseq[list1[i]], mseq1[i] );
-			for( i=0; i<ngroup2; i++ ) strcat( aseq[list2[i]], mseq2[i] );
+			newpos = strlen( aseq[rep] ); // kufuu?
+			for( i=0; i<ngroup0; i++ ) strcpy( aseq[list0[i]]+newpos, mseq0[i] );
+			for( i=0; i<ngroup1; i++ ) strcpy( aseq[list1[i]]+newpos, mseq1[i] );
+			for( i=0; i<ngroup2; i++ ) strcpy( aseq[list2[i]]+newpos, mseq2[i] );
+
+//			fprintf( stderr, "gapshift = %d\n", gapshift );
 		}
+		blocklen = 1 + countnogaplen( gaplen+j+1, gaplen+len0 );
+//		fprintf( stderr, "\nj=%d, blocklen=%d, len0=%d\n", j, blocklen, len0 );
+//		blocklen = 1;
+//		if( tmptmptmpmark ) exit( 1 );
 
 		newpos = strlen( aseq[rep] );
+
+#if 0
 		for( i=0; i<ngroup0; i++ ) aseq[list0[i]][newpos] = seq[list0[i]][j];
 		for( i=0; i<ngroup1; i++ ) aseq[list1[i]][newpos] = seq[list1[i]][posin12];
 		for( i=0; i<ngroup2; i++ ) aseq[list2[i]][newpos] = seq[list2[i]][posin12];
 		for( i=0; i<ngroup0; i++ ) aseq[list0[i]][newpos+1] = 0;
 		for( i=0; i<ngroup1; i++ ) aseq[list1[i]][newpos+1] = 0;
 		for( i=0; i<ngroup2; i++ ) aseq[list2[i]][newpos+1] = 0;
+#else
+
+		for( i=0; i<ngroup0; i++ )
+		{
+			lp = list0[i];
+			newchar = aseq[lp] + newpos;
+			strncpy0( newchar, seq[lp]+j, blocklen );
+		}
+		for( i=0; i<ngroup1; i++ )
+		{
+			lp = list1[i];
+			newchar = aseq[lp] + newpos;
+			strncpy0( newchar, seq[lp]+posin12, blocklen );
+		}
+		for( i=0; i<ngroup2; i++ )
+		{
+			lp = list2[i];
+			newchar = aseq[lp] + newpos;
+			strncpy0( newchar, seq[lp]+posin12, blocklen );
+		}
+//		fprintf( stderr, "aseq[l0] = %s\n", aseq[list0[0]] );
+//		fprintf( stderr, "aseq[l1] = %s\n", aseq[list1[0]] );
+//		fprintf( stderr, "aseq[l2] = %s\n", aseq[list2[0]] );
+//		exit( 1 );
+#endif
+
+//		fprintf( stderr, "j=%d -> %d\n", j, j+blocklen-1 );
+		j += (blocklen-1);
+
+
+		posin12 += (blocklen-1);
+
 
 		posin12++;
 	}
+#if 0
+	fprintf( stderr, "\n" );
+	fprintf( stderr, " seq[l0] = %s\n", aseq[list0[0]] );
+	fprintf( stderr, " seq[l1] = %s\n", aseq[list1[0]] );
+	fprintf( stderr, " seq[l2] = %s\n", aseq[list2[0]] );
+	fprintf( stderr, "=====>\n" );
+	fprintf( stderr, "aseq[l0] = %s\n", seq[list0[0]] );
+	fprintf( stderr, "aseq[l1] = %s\n", seq[list1[0]] );
+	fprintf( stderr, "aseq[l2] = %s\n", seq[list2[0]] );
+//if( tmptmptmpmark ) exit( 1 );
+#endif
 
 //	for( i=0; i<njob; i++ ) if( mar[i] != 3 ) strcpy( seq[i], aseq[i] );
 	for( i=0; i<ngroup0; i++ ) strcpy( seq[list0[i]], aseq[list0[i]] );
 	for( i=0; i<ngroup1; i++ ) strcpy( seq[list1[i]], aseq[list1[i]] );
 	for( i=0; i<ngroup2; i++ ) strcpy( seq[list2[i]], aseq[list2[i]] );
 
+
 	free( mar );
 	free( gaps );
 	free( list0 );
 	free( list1 );
 	free( list2 );
 	FreeCharMtx( mseq2 );
+	FreeCharMtx( mseq1 ); // ? added 2012/02/12
 	FreeCharMtx( mseq0 );
+	FreeCharMtx( aseq ); // ? added 2012/02/12
 }
 
 
-void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, int alloclen )
+void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, int alloclen, char gapchar )
 {
 	int *mar;
 	char *tmpseq;
@@ -320,7 +440,7 @@ void restorecommongaps( int njob, char **seq, int *ex1, int *ex2, int *gaplen, i
 		for( j=0; j<len1; j++ )
 		{
 			for( k=0; k<gaplen[j]; k++ )
-				*(cptr++) = '-';
+				*(cptr++) = gapchar; // ???
 			*(cptr++) = seq[i][j];
 		}
 		*cptr = 0;
