@@ -13,6 +13,7 @@ static int treeout;
 static int noalign;
 static int distout;
 static float lenfaca, lenfacb, lenfacc, lenfacd;
+static int tuplesize;
 #if 0
 #define PLENFACA 0.0123
 #define PLENFACB 10252
@@ -27,10 +28,14 @@ static float lenfaca, lenfacb, lenfacc, lenfacd;
 #define PLENFACB 10000
 #define PLENFACC 10000
 #define PLENFACD 0.1
-#define DLENFACA 0.01
-#define DLENFACB 2500
-#define DLENFACC 2500
-#define DLENFACD 0.1
+#define D6LENFACA 0.01
+#define D6LENFACB 2500
+#define D6LENFACC 2500
+#define D6LENFACD 0.1
+#define D10LENFACA 0.01
+#define D10LENFACB 1000000
+#define D10LENFACC 1000000
+#define D10LENFACD 0.0
 #endif
 
 #ifdef enablemultithread
@@ -119,6 +124,7 @@ void arguments( int argc, char *argv[] )
 	fftThreshold = NOTSPECIFIED;
 	TMorJTT = JTT;
 	scoreout = 0;
+	tuplesize = 6;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -198,9 +204,11 @@ void arguments( int argc, char *argv[] )
 				case 'T':
 					noalign = 1;
 					break;
+#if 0
 				case 'r':
 					fmodel = -1;
 					break;
+#endif
 				case 'D':
 					dorp = 'd';
 					break;
@@ -301,6 +309,10 @@ void arguments( int argc, char *argv[] )
 					fftWinSize = atoi( *++argv );
 					--argc;
 					goto nextoption;
+				case 'W':
+					tuplesize = atoi( *++argv );
+					--argc;
+					goto nextoption;
 				case 'Z':
 					checkC = 1;
 					break;
@@ -349,7 +361,7 @@ void seq_grp_nuc( int *grp, char *seq )
 			fprintf( stderr, "WARNING : Unknown character %c\r", *(seq-1) );
 	}
 	*grp = END_OF_VEC;
-	if( grp - grpbk < 6 )
+	if( grp - grpbk < tuplesize )
 	{
 //		fprintf( stderr, "\n\nWARNING: Too short.\nPlease also consider use mafft-ginsi, mafft-linsi or mafft-ginsi.\n\n\n" );
 //		exit( 1 );
@@ -378,7 +390,7 @@ void seq_grp( int *grp, char *seq )
 	}
 }
 
-void makecompositiontable_p( short *table, int *pointt )
+void makecompositiontable_p( int *table, int *pointt )
 {
 	int point;
 
@@ -386,12 +398,12 @@ void makecompositiontable_p( short *table, int *pointt )
 		table[point]++;
 }
 
-int commonsextet_p( short *table, int *pointt )
+int commonsextet_p( int *table, int *pointt )
 {
 	int value = 0;
-	short tmp;
+	int tmp;
 	int point;
-	static TLS short *memo = NULL;
+	static TLS int *memo = NULL;
 	static TLS int *ct = NULL;
 	static TLS int *cp;
 
@@ -407,10 +419,10 @@ int commonsextet_p( short *table, int *pointt )
 
 	if( !memo )
 	{
-		memo = (short *)calloc( tsize, sizeof( short ) );
+		memo = (int *)calloc( tsize, sizeof( int ) );
 		if( !memo ) ErrorExit( "Cannot allocate memo\n" );
-		ct = (int *)calloc( MIN( maxl, tsize )+1, sizeof( int ) );
-		if( !ct ) ErrorExit( "Cannot allocate memo\n" );
+		ct = (int *)calloc( MIN( maxl, tsize )+1, sizeof( int ) ); // chuui!!
+		if( !ct ) ErrorExit( "Cannot allocate ct\n" );
 	}
 
 	cp = ct;
@@ -430,6 +442,73 @@ int commonsextet_p( short *table, int *pointt )
 	return( value );
 }
 
+void makepointtable_nuc_dectet( int *pointt, int *n )
+{
+	int point;
+	register int *p;
+
+	if( *n == -1 )
+	{
+		*pointt = -1;
+		return;
+	}
+
+	p = n;
+	point  = *n++ *262144;
+	point += *n++ * 65536;
+	point += *n++ * 16384;
+	point += *n++ *  4096;
+	point += *n++ *  1024;
+	point += *n++ *   256;
+	point += *n++ *    64;
+	point += *n++ *    16;
+	point += *n++ *     4;
+	point += *n++;
+	*pointt++ = point;
+
+	while( *n != END_OF_VEC )
+	{
+		point -= *p++ *262144;
+		point *= 4;
+		point += *n++;
+		*pointt++ = point;
+
+	}
+	*pointt = END_OF_VEC;
+}
+
+void makepointtable_nuc_octet( int *pointt, int *n )
+{
+	int point;
+	register int *p;
+
+	if( *n == -1 )
+	{
+		*pointt = -1;
+		return;
+	}
+
+	p = n;
+	point  = *n++ * 16384;
+	point += *n++ *  4096;
+	point += *n++ *  1024;
+	point += *n++ *   256;
+	point += *n++ *    64;
+	point += *n++ *    16;
+	point += *n++ *     4;
+	point += *n++;
+	*pointt++ = point;
+
+	while( *n != END_OF_VEC )
+	{
+		point -= *p++ * 16384;
+		point *= 4;
+		point += *n++;
+		*pointt++ = point;
+	}
+	*pointt = END_OF_VEC;
+}
+
 void makepointtable_nuc( int *pointt, int *n )
 {
 	int point;
@@ -500,7 +579,7 @@ static void *distancematrixthread( void *arg )
 	int **pointt = targ->pointt;
 	float **mtx = targ->mtx;
 
-	short *table1;
+	int *table1;
 	int i, j;
 
 	while( 1 )
@@ -516,7 +595,7 @@ static void *distancematrixthread( void *arg )
 		*jobpospt = i+1;
 		pthread_mutex_unlock( targ->mutex );
 
-		table1 = (short *)calloc( tsize, sizeof( short ) );
+		table1 = (int *)calloc( tsize, sizeof( int ) );
 		if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
 		if( i % 10 == 0 )
 		{
@@ -1120,9 +1199,9 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 		if( mergeoralign[l] == '1' ) // jissainiha nai. atarashii hairetsu ha saigo dakara.
 		{
 			adjustgapmap( strlen( mseq2[0] )-len2nocommongap+len2, gapmap, mseq2[0] );
-			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
-			findnewgaps( clus2, mseq2, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg );
+			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
+			findnewgaps( clus2, 0, mseq2, gaplen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg, '-' );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][0][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1131,11 +1210,11 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 			adjustgapmap( strlen( mseq1[0] )-len1nocommongap+len1, gapmap, mseq1[0] );
 //			fprintf( stderr, ">STEP1 mseq1[0] = \n%s\n", mseq1[0] );
 //			fprintf( stderr, ">STEP1 mseq2[0] = \n%s\n", mseq2[0] );
-			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
+			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
 //			fprintf( stderr, "STEP2 mseq1[0] = %s\n", mseq1[0] );
 //			fprintf( stderr, "STEP2 mseq2[0] = %s\n", mseq2[0] );
-			findnewgaps( clus1, mseq1, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg );
+			findnewgaps( clus1, 0, mseq1, gaplen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg, '-' );
 //			fprintf( stderr, "STEP3 mseq1[0] = %s\n", mseq1[0] );
 //			fprintf( stderr, "STEP3 mseq2[0] = %s\n", mseq2[0] );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
@@ -1241,7 +1320,7 @@ int main( int argc, char *argv[] )
 	char *tmpseq;
 	int  **pointt;
 	float **mtx = NULL; // by D. Mathog
-	static short *table1;
+	static int *table1;
 	char b[B];
 	int ien;
 	double unweightedspscore;
@@ -1269,9 +1348,9 @@ int main( int argc, char *argv[] )
 	getnumlen( infp );
 	rewind( infp );
 
-	if( njob > 100000 )
+	if( njob > 1000000 )
 	{
-		fprintf( stderr, "The number of sequences must be < %d\n", 100000 );
+		fprintf( stderr, "The number of sequences must be < %d\n", 1000000 );
 		fprintf( stderr, "Please try the --parttree option for such large data.\n" );
 		exit( 1 );
 	}
@@ -1329,6 +1408,18 @@ int main( int argc, char *argv[] )
 
 	fprintf( stderr, "\n" );
 
+	fprintf( stderr, "tuplesize = %d, dorp = %c\n", tuplesize, dorp );
+	if( dorp == 'p' && tuplesize != 6 )
+	{
+		fprintf( stderr, "tuplesize must be 6 for aa sequence\n" );
+		exit( 1 );
+	}
+	if( dorp == 'd' && tuplesize != 6 && tuplesize != 10 )
+	{
+		fprintf( stderr, "tuplesize must be 6 or 10 for dna sequence\n" );
+		exit( 1 );
+	}
+
 	if( !treein )
 	{
 		fprintf( stderr, "\n\nMaking a distance matrix ..\n" );
@@ -1338,15 +1429,22 @@ int main( int argc, char *argv[] )
 		grpseq = AllocateIntVec( nlenmax+1 );
 		pointt = AllocateIntMtx( njob, nlenmax+1 );
     	mtx = AllocateFloatHalfMtx( njob ); 
-		if( dorp == 'd' ) tsize = (int)pow( 4, 6 );
+		if( dorp == 'd' ) tsize = (int)pow( 4, tuplesize );
 		else              tsize = (int)pow( 6, 6 );
 
-		if( dorp == 'd' )
+		if( dorp == 'd' && tuplesize == 6 )
 		{
-			lenfaca = DLENFACA;
-			lenfacb = DLENFACB;
-			lenfacc = DLENFACC;
-			lenfacd = DLENFACD;
+			lenfaca = D6LENFACA;
+			lenfacb = D6LENFACB;
+			lenfacc = D6LENFACC;
+			lenfacd = D6LENFACD;
+		}
+		else if( dorp == 'd' && tuplesize == 10 )
+		{
+			lenfaca = D10LENFACA;
+			lenfacb = D10LENFACB;
+			lenfacc = D10LENFACC;
+			lenfacd = D10LENFACD;
 		}
 		else    
 		{
@@ -1371,7 +1469,17 @@ int main( int argc, char *argv[] )
 			if( dorp == 'd' ) /* nuc */
 			{
 				seq_grp_nuc( grpseq, tmpseq );
-				makepointtable_nuc( pointt[i], grpseq );
+//				makepointtable_nuc( pointt[i], grpseq );
+//				makepointtable_nuc_octet( pointt[i], grpseq );
+				if( tuplesize == 10 )
+					makepointtable_nuc_dectet( pointt[i], grpseq );
+				else if( tuplesize == 6 )
+					makepointtable_nuc( pointt[i], grpseq );
+				else
+				{
+					fprintf( stderr, "tuplesize=%d: not supported\n", tuplesize );
+					exit( 1 );
+				}
 			}
 			else                 /* amino */
 			{
@@ -1417,7 +1525,7 @@ int main( int argc, char *argv[] )
 		{
 			for( i=0; i<njob; i++ )
 			{
-				table1 = (short *)calloc( tsize, sizeof( short ) );
+				table1 = (int *)calloc( tsize, sizeof( int ) );
 				if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
 				if( i % 10 == 0 )
 				{
@@ -1451,9 +1559,10 @@ int main( int argc, char *argv[] )
 					longer=(float)nogaplen[j];
 					shorter=(float)nogaplen[i];
 				}
-//				lenfac = 3.0 / ( LENFACA + LENFACB / ( longer + LENFACC ) + shorter / longer * LENFACD );
+//				if( tuplesize == 6 )
 				lenfac = 1.0 / ( shorter / longer * lenfacd + lenfacb / ( longer + lenfacc ) + lenfaca );
-//				lenfac = 1.0;
+//				else
+//					lenfac = 1.0;
 //				fprintf( stderr, "lenfac = %f (%.0f,%.0f)\n", lenfac, longer, shorter );
 				bunbo = MIN( mtx[i][0], mtx[j][0] );
 				if( bunbo == 0.0 )
@@ -1520,7 +1629,7 @@ int main( int argc, char *argv[] )
 	else
 	{
 		fprintf( stderr, "Constructing a UPGMA tree ... " );
-		fixed_musclesupg_float_realloc_nobk_halfmtx( njob, mtx, topol, len, dep );
+		fixed_musclesupg_float_realloc_nobk_halfmtx( njob, mtx, topol, len, dep, 1 );
 		FreeFloatHalfMtx( mtx, njob );
 	}
 //	else 
