@@ -25,6 +25,7 @@ extern void countnode( int nseq, int ***topol, double **node );
 extern void countnode_int( int nseq, int ***topol, int **node );
 extern void counteff_simple( int nseq, int ***topol, double **len, double *node );
 extern void counteff_simple_float( int nseq, int ***topol, float **len, double *node );
+extern void counteff_simple_float_nostatic( int nseq, int ***topol, float **len, double *node );
 extern void counteff( int nseq, int ***topol, double **len, double **node );
 extern float score_calc1( char *seq1, char *seq2 );
 extern float score_calcp( char *seq1, char *seq2, int len );
@@ -204,6 +205,7 @@ extern void writeData( FILE *fp, int locnjob, char name[][B], int nlen[], char *
 extern void writeData_pointer( FILE *fp, int locnjob, char **name, int *nlen, char **aseq );
 extern void readhat2_floathalf( FILE *fp, int nseq, char name[M][B], float **mtx );
 extern void readhat2_floathalf_pointer( FILE *fp, int nseq, char **name, float **mtx );
+extern void readhat2_floathalf_part_pointer( FILE *fp, int nseq, int nadd, char **name, float **mtx );
 extern void readhat2_float( FILE *fp, int nseq, char name[M][B], float **mtx );
 extern void readhat2_int( FILE *fp, int nseq, char name[M][B], int **mtx );
 extern void readhat2_pointer( FILE *fp, int nseq, char **name, double **mtx );
@@ -213,6 +215,7 @@ extern void WriteFloatHat2( FILE *hat2p, int locnjob, char name[M][B], float **m
 extern void WriteHat2_int( FILE *hat2p, int locnjob, char name[M][B], int **mtx );
 extern void WriteHat2( FILE *hat2p, int locnjob, char name[M][B], double **mtx );
 extern void WriteHat2_pointer( FILE *hat2p, int locnjob, char **name, double **mtx );
+extern void WriteHat2_part_pointer( FILE *hat2p, int locnjob, int nadd, char **name, double **mtx );
 extern int ReadFasta_sub( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadSsearch( FILE *fp, double *dis, int nseq, char name[M][B] );
 extern int ReadBlastm7( FILE *fp, double *dis, int qmem, char name[M][B], LocalHom *localhomlist );
@@ -289,7 +292,7 @@ extern void foldalignedrna( int clus1, int clus2, char **mseq1, char **mseq2, do
 void readmccaskill( FILE *fp, RNApair **pairprob, int length );
 void makegrouprna( RNApair ***group, RNApair ***all, int *memlist );
 void makegrouprnait( RNApair ***group, RNApair ***all, char **pair, int s );
-extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep * );
+extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep *, int progressout );
 extern void loadtree( int nseq, int ***topol, float **len, char **name, int *nlen, Treedep * );
 extern void loadtop( int nseq, float **eff, int ***topol, float **len );
 extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep * );
@@ -309,11 +312,13 @@ extern void getnumlen_nogap_outallreg_web( FILE *fp, FILE *ofp, int *nlenminpt,
 extern void getnumlen_nogap_outallreg( FILE *fp, int *nlenminpt );
 extern double plainscore( int nseq, char **seq );
 extern void eq2dash( char *s );
-extern void findnewgaps( int n, char **seq, int *gaplen );
+extern void findnewgaps( int n, int rep, char **seq, int *gaplen );
 extern void findcommongaps( int, char **, int * );
 extern void adjustgapmap( int, int *, char * );
-extern void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int *gapmap, int alloclen, char alg );
-extern void restorecommongaps( int n, char **seq, int *top0, int *top1, int *gaplen, int alloclen );
+extern void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int *gapmap, int alloclen, char alg, char gapchar );
+extern void restorecommongaps( int n, char **seq, int *top0, int *top1, int *gaplen, int alloclen, char gapchar );
 extern int samemember( int *mem, int *cand );
 extern int includemember( int *mem, int *cand );
 extern void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **aln2, int alloclen, char alg );
+extern void sreverse( char *r, char *s );
+extern void addonetip( int njobc, int ***topolc, float **lenc, float **iscorec, int ***topol, float **len, Treedep *dep, int treeout, int iadd, char **name );
