@@ -124,7 +124,10 @@ static void *athread( void *arg )
 
 		sprintf( com, "tr -d '\\r' < %s/_mccaskillinorg > %s/_mccaskillin", dirname, dirname );
 		system( com ); // for cygwin, wakaran
-		sprintf( com, "cd %s; %s/mxscarnamod -m -writebpp  _mccaskillin > _mccaskillout 2>_dum", dirname, whereismccaskillmea );
+		if( alg == 'G' )
+			sprintf( com, "cd %s; %s/dafs --mafft-out _mccaskillout _mccaskillin > _dum1 2>_dum", dirname, whereismccaskillmea );
+		else
+			sprintf( com, "cd %s; %s/mxscarnamod -m -writebpp  _mccaskillin > _mccaskillout 2>_dum", dirname, whereismccaskillmea );
 		res = system( com );
 
 		if( res )
@@ -161,6 +164,7 @@ void arguments( int argc, char *argv[] )
 	kimuraR = NOTSPECIFIED;
 	pamN = NOTSPECIFIED;
 	whereismccaskillmea = NULL;
+	alg = 's';
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -183,6 +187,12 @@ void arguments( int argc, char *argv[] )
 					fprintf( stderr, "nthread = %d\n", nthread );
 					--argc; 
 					goto nextoption;
+				case 's':
+					alg = 's'; // use scarna; default
+					break;
+				case 'G':
+					alg = 'G'; // use dafs, instead of scarna
+					break;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -285,7 +295,10 @@ int main( int argc, char *argv[] )
 
 	constants( njob, seq );
 
-	fprintf( stderr, "Running mxscarna with the mccaskill_mea mode.\n" );
+	if( alg == 'G' )
+		fprintf( stderr, "Running DAFS (Sato et al. 2012; http://www.ncrna.org/).\n" );
+	else
+		fprintf( stderr, "Running mxscarna with the mccaskill_mea mode.\n" );
 #ifdef enablemultithread
 	if( nthread > 0 )
 	{
@@ -342,7 +355,10 @@ int main( int argc, char *argv[] )
 			fclose( infp );
 	
 			system( "tr -d '\\r' < _mccaskillinorg > _mccaskillin" ); // for cygwin, wakaran
-			sprintf( com, "env PATH=%s mxscarnamod -m -writebpp  _mccaskillin > _mccaskillout 2>_dum", whereismccaskillmea );
+			if( alg == 'G' )
+				sprintf( com, "env PATH=%s dafs --mafft-out _mccaskillout _mccaskillin > _dum1 2>_dum", whereismccaskillmea );
+			else
+				sprintf( com, "env PATH=%s mxscarnamod -m -writebpp  _mccaskillin > _mccaskillout 2>_dum", whereismccaskillmea );
 			res = system( com );
 	
 			if( res )
