@@ -20,7 +20,7 @@
 #include <pthread.h>
 #endif
 
-#define VERSION "6.864b"
+#define VERSION "6.940b"
 #define SHOWVERSION fprintf( stderr, "%s (%s) Version " VERSION " alg=%c, model=%s\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":"aa", alg, modelname, nthread )
 
 #define FFT_THRESHOLD  80
@@ -29,7 +29,7 @@
 #define DISPSEQF  60
 #define DISPSITEI    0
 #define MAXITERATION 500
-#define M   50000       /* njob no saidaiti */
+#define M  500000       /* njob no saidaiti */
 #define N 5000000       /* nlen no saidaiti */
 #define MAXSEG 100000
 #define B     256
@@ -271,6 +271,7 @@ typedef struct _Treedep
 	int child0;
 	int child1;
 	int done;
+	float distfromtip;
 } Treedep;
 
 
@@ -299,4 +300,4 @@ extern int parallelizationstrategy;
 extern int scoreout;
 extern int outnumber;
 
-extern char *newgapstr;
+extern TLS char *newgapstr;
