@@ -6,6 +6,8 @@
 
 static int usecache;
 static char *whereispairalign;
+static char *odir;
+static char *pdir;
 static double scale;
 static int *alreadyoutput;
 static int equivthreshold;
@@ -404,10 +406,10 @@ static float calltmalign( char **mseq1, char **mseq2, double *equiv, char *fname
 	return( value );
 }
 
-static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
+static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *equiv, char *fname1, char *fname2, int alloclen )
 {
 	FILE *fp;
-	int res;
+//	int res;
 	static char com[10000];
 	float value;
 	char cachedir[10000];
@@ -420,28 +422,31 @@ static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *e
 
 	if( usecache )
 	{
-		sprintf( cachedir, "%s/.rashoutcache", getenv( "HOME" ) );
-		sprintf( com, "mkdir -p %s", cachedir );
-		system( com );
+//		sprintf( cachedir, "tmp" );
+		sprintf( cachedir, "%s", pdir );
 
-		sprintf( cachefile, "%s/%s%s-%s%s", cachedir, fname1, chain1, fname2, chain2 );
+		sprintf( cachefile, "%s/%s.%s.rash", cachedir, fname1, fname2 );
 
 		runnow = 0;
 		fp = fopen( cachefile, "r" );
-		if( fp == NULL ) runnow = 1;
+		if( fp == NULL )
+		{
+			fprintf( stderr, "Cannot open %s\n", cachefile );
+			exit( 1 );
+		}
 		else
 		{
-			fgets( com, 100, fp );
-			if( strncmp( com, "successful", 10 ) ) runnow = 1;
 			fclose( fp );
 		}
 	}
 	else
 	{
-		runnow = 1;
+		fprintf( stderr, "Not supported!\n" );
+		exit( 1 );
 	}
 
-	if( runnow )
+#if 0
+	if( 0 )
 	{
 #if 0
 		sprintf( com, "ln -s %s %s.pdb 2>_dum", fname1, fname1 );
@@ -473,18 +478,17 @@ static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *e
 	
 	}
 	else
+#endif
 	{
-		fprintf( stderr, "Use cache!\n" );
-		sprintf( com, "grep -v successful %s > %s.rashout", cachefile, pairid );
+		fprintf( stderr, "Use cache! cachefile = %s\n", cachefile );
+		sprintf( com, "cat %s > %s.rashout", cachefile, pairid );
 		system( com );
 	}
 
 	if( usecache && runnow )
 	{
-		sprintf( com, "echo successful >  %s", cachefile );
-		system( com );
-		sprintf( com, "cat %s.rashout >>  %s", pairid, cachefile );
-		system( com );
+		fprintf( stderr, "Okashii! usechache=%d, runnow=%d\n", usecache, runnow );
+		exit( 1 );
 	}
 
 	sprintf( com, "%s.rashout", pairid );
@@ -615,7 +619,7 @@ static void preparetmalign( FILE *fp, char ***strfiles, char ***chainids, char *
 	}
 }
 
-static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***seqpt, char ***mseq1pt, char ***mseq2pt, double **equivpt, int *alloclenpt )
+static void prepareash( FILE *fp, char *inputfile, char ***strfiles, char ***chainids, char ***seqpt, char ***mseq1pt, char ***mseq2pt, double **equivpt, int *alloclenpt )
 {
 	int i, res;
 	char *dumseq;
@@ -623,11 +627,13 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	char fname[1000];
 	char command[1000];
 	int linenum, istr, nstr;
-	FILE *checkfp;
+//	FILE *checkfp;
 	char *sline; 
 	int use[1000];
 	linenum = 0;
 	nstr = 0;
+
+	fprintf( stderr, "inputfile = %s\n", inputfile );
 	while( 1 )
 	{
 		fgets( line, 999, fp );
@@ -641,20 +647,13 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 			continue;
 		}
 		extractfirstword( sline );
+#if 0
 		checkfp = fopen( sline, "r" );
 		if( checkfp == NULL )
 		{
 			fprintf( stderr, "Cannot open %s.\n", sline );
 			exit( 1 );
 		}
-#if 0
-		fgets( linec, 999, checkfp );
-		if( strncmp( "HEADER ", linec, 7 ) )
-		{
-			fprintf( stderr, "Check the format of %s.\n", sline );
-			exit( 1 );
-		}
-#endif
 		if( checkcbeta( checkfp ) ) 
 		{
 			fprintf( stderr, "%s has no C-beta atoms.\n", sline );
@@ -663,6 +662,9 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 		else
 			nstr++;
 		fclose( checkfp );
+#else
+		nstr++;
+#endif
 		linenum++;
 	}
 	njob = nstr;
@@ -679,6 +681,7 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 		fgets( line, 999, fp );
 		if( feof( fp ) ) break;
 		sline = strip( line );
+		fprintf( stderr, "sline = %s\n", sline );
 		if( use[linenum++] ) 
 		{
 			(*chainids)[istr][0] = getchainid( sline );
@@ -686,6 +689,7 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 			extractfirstword( sline );
 			sprintf( fname, "%s", sline );
 			cutpath( fname );
+#if 0
 			sprintf( command, "cp %s %s.pdb", sline, fname );
 			system( command );
 			sprintf( command, "perl \"%s/clean.pl\" %s.pdb", whereispairalign, fname );
@@ -695,6 +699,7 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 				fprintf( stderr, "error: Install clean.pl\n" );
 				exit( 1 );
 			}
+#endif
 			strcpy( (*strfiles)[istr++], fname );
 		}
 	}
@@ -708,6 +713,18 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	alreadyoutput = AllocateIntVec( njob );
 	for( i=0; i<njob; i++ ) alreadyoutput[i] = 0;
 
+	fprintf( stderr, "Running pdp_ash_batch.pl..\n" );
+//	sprintf( command, "/opt/protein/share/domains/code/pdp_ash/pdp_ash_batch.pl -f %s -d tmp -i %d", inputfile, wheretooutput  );
+	sprintf( command, "/opt/protein/share/mafftash/pdp_ash/pdp_ash_batch.pl -f %s -d %s -i %s", inputfile, pdir, odir );
+	res = system( command );
+	if( res )
+	{
+		fprintf( stderr, "Ask KM!\n" );
+		exit( 1 );
+	}
+	fprintf( stderr, "done\n" );
+
+
 	for( i=0; i<istr; i++ )
 	{
 		fprintf( stderr, "i=%d\n", i );
@@ -716,8 +733,8 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 		(*mseq1pt)[0] = (*seqpt)[i];
 		(*mseq2pt)[0] = dumseq;
 
-		callrash( i, i, *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
-		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
+		callrash( i, i, *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*strfiles)[i], *alloclenpt );
+		fprintf( stdout, ">%d_%s\n%s\n", i+1, (*strfiles)[i], (*seqpt)[i] );
 		alreadyoutput[i] = 1;
 	}
 }
@@ -776,6 +793,8 @@ void arguments( int argc, char *argv[] )
 	fftThreshold = NOTSPECIFIED;
 	RNAppenalty = NOTSPECIFIED;
 	RNApthr = NOTSPECIFIED;
+	odir = "";
+	pdir = "";
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -838,6 +857,16 @@ void arguments( int argc, char *argv[] )
 					fprintf( stderr, "whereispairalign = %s\n", whereispairalign );
 					--argc; 
 					goto nextoption;
+				case 'o':
+					odir = *++argv;
+					fprintf( stderr, "odir = %s\n", odir );
+					--argc; 
+					goto nextoption;
+				case 'p':
+					pdir = *++argv;
+					fprintf( stderr, "pdir = %s\n", pdir );
+					--argc; 
+					goto nextoption;
 				case 't':
 					equivthreshold = atoi( *++argv );
 					--argc;
@@ -1109,7 +1138,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 						break;
 					case( 'R' ):
 						fprintf( stderr, "  Calling PDP_ASH.pl %d-%d/%d    \r", i+1, j+1, njob );
-						pscore = callrash( i, j, mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
+						pscore = callrash( i, j, mseq1, mseq2, equiv, strfiles[i], strfiles[j], alloclen );
 						off1 = off2 = 0;
 						break;
 					ErrorExit( "ERROR IN SOURCE FILE" );
@@ -1128,7 +1157,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 			{
 				alreadyoutput[i] = 1;
 				gappick0( seq[i], mseq1[0] );
-				fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, strfiles[i], chainids[i], seq[i] );
+				fprintf( stdout, ">%d_%s\n%s\n", i+1, strfiles[i], seq[i] );
 				strcpy( checkseq[i], seq[i] );
 			}
 			else
@@ -1328,7 +1357,7 @@ int main( int argc, char *argv[] )
 	nlenmax = 10000; // tekitou
 
 	if( alg == 'R' )
-		prepareash( infp, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
+		prepareash( infp, inputfile, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
 	else if( alg == 'T' )
 		preparetmalign( infp, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
 
