@@ -2346,7 +2346,7 @@ exit( 1 );
 		{
 			fprintf( stderr, "upgma       " );
 //			veryfastsupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len );
-			fixed_musclesupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len, NULL );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len, NULL, 1 );
 			fprintf( stderr, "\r                      \r" );
 		}
 		else
