@@ -10,14 +10,15 @@ static int topin;
 static int treeout;
 static int distout;
 static int noalign;
+static int multidist;
 
+#ifdef enablemultithread
 typedef struct _jobtable
 {
     int i;  
     int j;  
 } Jobtable;
 
-#ifdef enablemultithread
 typedef struct _distancematrixthread_arg
 {
 	int njob;
@@ -110,6 +111,7 @@ void arguments( int argc, char *argv[] )
 	TMorJTT = JTT;
 	consweight_multi = 1.0;
 	consweight_rna = 0.0;
+	multidist = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -303,9 +305,14 @@ void arguments( int argc, char *argv[] )
 				case 'v':
 					tbrweight = 3;
 					break;
+				case 'd':
+					multidist = 1;
+					break;
+#if 0
 				case 'd':
 					disp = 1;
 					break;
+#endif
 /* Modified 01/08/27, default: user tree */
 				case 'J':
 					tbutree = 0;
@@ -1113,9 +1120,9 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 		if( mergeoralign[l] == '1' ) // jissainiha nai. atarashii hairetsu ha saigo dakara.
 		{
 			adjustgapmap( strlen( mseq2[0] )-len2nocommongap+len2, gapmap, mseq2[0] );
-			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
-			findnewgaps( clus2, mseq2, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg );
+			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
+			findnewgaps( clus2, 0, mseq2, gaplen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg, '-' );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][0][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1124,9 +1131,9 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 //			fprintf( stderr, ">mseq1[0] = \n%s\n", mseq1[0] );
 //			fprintf( stderr, ">mseq2[0] = \n%s\n", mseq2[0] );
 			adjustgapmap( strlen( mseq1[0] )-len1nocommongap+len1, gapmap, mseq1[0] );
-			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
-			findnewgaps( clus1, mseq1, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg );
+			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
+			findnewgaps( clus1, 0, mseq1, gaplen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg, '-' );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][1][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1472,12 +1479,31 @@ int main( int argc, char *argv[] )
 		}
 		else
 		{
-			fprintf( stderr, "Loading 'hat2' ... " );
-			prep = fopen( "hat2", "r" );
-			if( prep == NULL ) ErrorExit( "Make hat2." );
-			readhat2_floathalf_pointer( prep, njob, name, iscore );
-			fclose( prep );
-			fprintf( stderr, "done.\n" );
+			if( multidist )
+			{
+				fprintf( stderr, "Loading 'hat2n' (aligned sequences - new sequences) ... " );
+				prep = fopen( "hat2n", "r" );
+				if( prep == NULL ) ErrorExit( "Make hat2." );
+				readhat2_floathalf_pointer( prep, njob, name, iscore );
+				fclose( prep );
+				fprintf( stderr, "done.\n" );
+			
+				fprintf( stderr, "Loading 'hat2i' (aligned sequences) ... " );
+				prep = fopen( "hat2i", "r" );
+				if( prep == NULL ) ErrorExit( "Make hat2i." );
+				readhat2_floathalf_pointer( prep, njob-nadd, name, iscore );
+				fclose( prep );
+				fprintf( stderr, "done.\n" );
+			}
+			else
+			{
+				fprintf( stderr, "Loading 'hat2' ... " );
+				prep = fopen( "hat2", "r" );
+				if( prep == NULL ) ErrorExit( "Make hat2." );
+				readhat2_floathalf_pointer( prep, njob, name, iscore );
+				fclose( prep );
+				fprintf( stderr, "done.\n" );
+			}
 		}
 #if 1
 		if( distout )
@@ -1520,7 +1546,7 @@ int main( int argc, char *argv[] )
 		}
 		else
 		{
-			fixed_musclesupg_float_realloc_nobk_halfmtx( njob, iscore, topol, len, dep );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( njob, iscore, topol, len, dep, 1 );
 		}
 //		else 
 //			ErrorExit( "Incorrect tree\n" );
@@ -1530,7 +1556,7 @@ int main( int argc, char *argv[] )
 //			for( i=0; i<nkozo-1; i++ )
 //				for( j=i+1; j<nkozo; j++ )
 //					fprintf( stderr, "iscore_kozo[%d][%d] =~ %f\n", i, j, iscore_kozo[i][j-i] );
-			fixed_musclesupg_float_realloc_nobk_halfmtx( nkozo, iscore_kozo, topol_kozo, len_kozo, NULL );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( nkozo, iscore_kozo, topol_kozo, len_kozo, NULL, 1 );
 		}
 		fprintf( stderr, "\ndone.\n\n" );
 		fflush( stderr );
@@ -1569,7 +1595,15 @@ int main( int argc, char *argv[] )
 		utree = 0; counteff( njob, topol, len, eff ); utree = 1;
 #else
 		counteff_simple_float( njob, topol, len, eff );
-
+		for( i=njob-nadd; i<njob; i++ ) eff[i] /= (double)100;
+#if 0
+		fprintf( stderr, "######  WEIGHT = \n" );
+		for( i=0; i<njob; i++ )
+		{
+			fprintf( stderr, "w[%d] = %f\n", i, eff[i] );
+		}
+		exit( 1 );
+#endif
 		if( nkozo )
 		{
 //			counteff_simple_float( nkozo, topol_kozo, len_kozo, eff_kozo ); // single weight nanode iranai
