@@ -302,14 +302,6 @@ void mseqcat( char **seq1, char **seq2, double **eff, double *effarr1, double *e
 
 	
 
-void strnbcat( char *s1, char *s2, int m )   /* s1 + s2 ---> s2  */
-{
-	static char b[N];
-	strncpy( b, s1, m ); b[m] = 0;
-	strcat( b, s2 );
-	strcpy( s2, b );
-}
-
 #if 0
 int conjuction( char pair[njob][njob], int s, char **seq, char **aseq, double *peff, double *eff, char name[M][B], char aname[M][B], char *d )
 #else
@@ -600,9 +592,10 @@ int conjuctionfortbfast_kozo( double *tmptmptmp, char **pair, int s, char **seq,
 int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char *d )
 {
 	int m, k;
-	char b[B];
+	char *b;
 	double total;
 
+	b = calloc( B, sizeof( char ) );
 #if DEBUG
 	fprintf( stderr, "s = %d\n", s );
 #endif
@@ -632,9 +625,10 @@ int conjuctionfortbfast( char **pair, int s, char **seq, char **aseq, double *pe
 	for( m=0; m<k; m++ )
 		peff[m] /= total;
 #endif
+	free( b );
 	return( k );
 }
-int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char name[M][B], char aname[M][B], char *d )
+int conjuction( char **pair, int s, char **seq, char **aseq, double *peff, double *eff, char **name, char **aname, char *d )
 {
 	int m, k;
 	char b[B];
