@@ -53,6 +53,7 @@ typedef struct _threadarg
 } threadarg_t;
 #endif
 
+#if 1
 static void shuffle( int *arr, int n )
 {
 	int i;
@@ -70,6 +71,7 @@ static void shuffle( int *arr, int n )
 		}
 	}
 }
+#endif
 
 static void Writeoption2( FILE *fp, int cycle, double cut )
 {
@@ -1276,7 +1278,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 #endif
 	static char *indication1, *indication2;
 	static LocalHom ***localhomshrink = NULL;
-	float impmatch, oimpmatch;
+	float impmatch = 0.0, oimpmatch = 0.0;
 	static int *gapmap1;
 	static int *gapmap2;
 	double tmpdouble;
